#include "main.h"
#include "global.h"
#include "functions.h"

int main()
{
    allegro_init(); // Initialize Allegro
    set_window_title("MCLevelEdit");
    install_keyboard(); // Keyboard input
    request_refresh_rate(75);
    set_color_depth(16);
    if(set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, 640, 480, 0, 0))
    {
       set_color_depth(15);
       if( set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, 640, 480, 0, 0))
       {
           allegro_message("Couldn't set graphics mode at _all_.  (Attempted to set 15bit and 16bit color at 640x480)");
           return -1;
       }
    }
    set_display_switch_mode(SWITCH_PAUSE); // Make program pause when user ALT+TABs
    text_mode(-1); // Make displayed text transparent

    // Time for the datafile!
    data = load_datafile("data.dat"); // Loads the data.dat datafile
    if(!data) // Error checking
    {
        allegro_message("Couldn't load data.dat!  Perhaps you renamed or moved it?"); // Pop-Up error
        return(-1); // End the program
    }
    tile[0] = (RLE_SPRITE *)data[grass].dat;
    tile[1] = (RLE_SPRITE *)data[desert].dat;
    tile[2] = (RLE_SPRITE *)data[water].dat;
    tile[3] = (RLE_SPRITE *)data[happy].dat;
    tile[4] = (RLE_SPRITE *)data[sad].dat;
    tile[5] = (RLE_SPRITE *)data[surprised].dat;
    tile[6] = (RLE_SPRITE *)data[kiss].dat;
    tile[7] = (RLE_SPRITE *)data[angry].dat;
    tile[8] = (RLE_SPRITE *)data[undecided].dat;
    tile[9] = (RLE_SPRITE *)data[embarassed].dat;
    tile[10] = (RLE_SPRITE *)data[angel].dat;
    tile[11] = (RLE_SPRITE *)data[cool].dat;
    tile[12] = (RLE_SPRITE *)data[tongue].dat;
    tile[13] = (RLE_SPRITE *)data[money].dat;
    tile[14] = (RLE_SPRITE *)data[book].dat;
    tile[15] = (RLE_SPRITE *)data[console].dat;
    tile[16] = (RLE_SPRITE *)data[crazy].dat;
    tile[17] = (RLE_SPRITE *)data[disks].dat;
    tile[18] = (RLE_SPRITE *)data[edit].dat;
    tile[19] = (RLE_SPRITE *)data[eyes].dat;
    tile[20] = (RLE_SPRITE *)data[folder].dat;
    tile[21] = (RLE_SPRITE *)data[world].dat;

    for(int i=23; i<=256; i++)
    {
        tile[i-1] = (RLE_SPRITE *)data[i].dat;
    }

    buffer = create_bitmap(SCREEN_W, 447); // Create the buffer!
    tilebuffer = create_bitmap(SCREEN_W, 33); // Tile selection buffer
    
    install_timer(); // Timing functions
    install_mouse();
    
    set_alpha_blender();
    
    load_map("level.mle");

    bool goingright = false;
    bool goingleft = false;
    bool goingup = false;
    bool goingdown = false;

    while(true)
    {
        if(key[KEY_ESC]) // Hit Escape?  Quit the game!
        {
            save_map("level.mle");
            break;
        }
        if(key[KEY_F1]) break;
        if(key[KEY_RIGHT]) install_int_ex(selection_right, MSEC_TO_TIMER(50));
        if(!key[KEY_RIGHT]) remove_int(selection_right);
        if(key[KEY_LEFT]) install_int_ex(selection_left, MSEC_TO_TIMER(50));
        if(!key[KEY_LEFT]) remove_int(selection_left);
        if(key[KEY_S]) save_map("level.mle");
        
        clear(buffer); // Clear the buffer!
        clear(tilebuffer);
        clear_keybuf(); // Get rid of old keypresses

        if(mouse_x == SCREEN_W-1){ if( scrollx > -(xtotal*XTILESIZE - 640)){ scrollx-=4; goingright = true; goingleft = false; /* goingup = false; */ } }        
        if(mouse_x == 0){ if(scrollx < 0){ scrollx+=4; goingleft = true; goingright = false; /* goingup = false; */ } }
        if(mouse_y == SCREEN_H-1){ if(scrolly > -(ytotal*YTILESIZE - 448)){ scrolly-=4; goingdown = true; goingup = false; /* goingleft = false; goingright = false; */ } }
        if(mouse_y == 0){ if(scrolly < 0){ scrolly+=4; goingup = true; goingdown = false; /* goingleft = false; goingright = false; */ } }
        
        if(goingright) if(scrollx%XTILESIZE!=0) scrollx-=4;
        if(goingleft) if(scrollx%XTILESIZE!=0) scrollx+=4;
        if(goingdown) if(scrolly%YTILESIZE!=0) scrolly-=4;
        if(goingup) if(scrolly%YTILESIZE!=0) scrolly+=4;
        
        if(mouse_b & 1){ change_tile(); select_tile(); }
        if(mouse_b & 2){ rightclick(); }
        
        draw_map();
        
        draw_tiles();
        
        set_trans_blender(0, 0, 0, 180);
        if(!rightclicked) draw_trans_rle_sprite(tilebuffer, (RLE_SPRITE *)data[selected].dat, wheretodraw, 0);
        
        if(mouse_y <= 432) show_mouse(buffer);
        
        vsync();
        blit(buffer, screen, 0, 0, 0, 0, buffer->w, buffer->h);
        blit(tilebuffer, screen, 0, 0, 0, 448, buffer->w, buffer->h);
        if(mouse_y >= 433) show_mouse(screen);
    }
    
    return 0;
}
END_OF_MAIN();

